#!/bin/python

from setuptools import setup

setup(  name        = "roomai",
        version     = "0.1.1",
        description = "A toolkit for developing and comparing imperfect information game bots",
        url         = "https://github.com/roomai/RoomAI",
        author      = "algorithmdog",
        author_email= "lili1987mail@gmail.com",
        license     = "MIT",
        packages    = ["roomai","roomai.kuhn","roomai.common","roomai.texas","roomai.fivecardstud","roomai.sevenking","roomai.doudizhu"],
        zip_safe    = False)



#!/bin/python
import roomai.texas
import random
import roomai.common
random.seed(4)
class Player(roomai.common.AbstractPlayer):
    """
    """
    def receive_info(self, info):
        """

        Args:
            info:
        """
        available_actions = info
    def take_action(self):
        """

        Returns:

        """
        action = raw_input("choosed_acton:")
        #action = ""
        return roomai.texas.TexasHoldemAction.lookup(action)
    def reset(self):
        """

        """
        pass


def show_public(public_state):
    """

    Args:
        public_state:
    """
    print ("dealer_id:%d\n"%(public_state.dealer_id) +\
           "chips:%s"%(",".join([str(i) for i in public_state.chips])) +\
           "  bets:%s"%(",".join([str(i) for i in public_state.bets])) +\
           "  is_folds:%s\n"%(",".join([str(i) for i in public_state.is_fold])) +\
           "public_cards:%s"%(",".join([c.key for c in public_state.public_cards]))
           )
def show_info(info):
    """

    Args:
        info:
    """
    person_state          = info.person_state
    print ("%d available_actions: %s"%(person_state.id, ",".join(sorted(person_state.available_actions.keys()))))
    print ("%d cards:%s"%(person_state.id,",".join([c.key for c in person_state.hand_cards])))

if __name__ == "__main__":
    players     = [Player(), Player(), Player()]
    env         = roomai.texas.TexasHoldemEnv()

    num_players = len(players)
    infos, public_state, person_states, private_state = env.init({"num_players": num_players})
    show_public(public_state)
    for i in range(env.num_players):
        players[i].receive_info(infos[i])
        show_info(infos[i])
    print "\n"


    while public_state.is_terminal == False:
        turn = public_state.turn
        action = players[turn].take_action()
        print "%d player take an action (%s)"%(turn,action.key)
        infos, public_state, person_states, private_state = env.forward(action)
        show_public(public_state)
        for i in range(env.num_players):
            players[i].receive_info(infos[i])
            show_info(infos[i])
        print "\n"

    print (public_state.scores)



#!/bin/python
#coding:utf-8
import random
import roomai.common
import sys

class TexasHoldemRandomPlayer(roomai.common.AbstractPlayer):
    """
    """
    def __init__(self):
        """

        """
        self.available_actions = None
        self.info              = None
           
    def receive_info(self, info):
        """

        Args:
            info:
        """
        self.info              = info
        self.available_actions = info.person_state.available_actions

    def take_action(self):

        '''
        print "\n\n\n"
        if self.info.public_state.previous_id is not None:
            print "previous_id", self.info.public_state.previous_id
        if self.info.public_state.previous_action is not None:
            print "previous_action", self.info.public_state.previous_action.key()
        print "stage", self.info.public_state.stage
        print "is_fold", self.info.public_state.is_fold, self.info.public_state.num_quit
        print "is_allin", self.info.public_state.is_allin, self.info.public_state.num_allin
        print "is_needed_action",self.info.public_state.is_needed_to_action, self.info.public_state.num_needed_to_action
        print "turn:",self.info.public_state.turn
        print "chips:",self.info.public_state.chips
        print "bets:",self.info.public_state.bets
        print "max_bet_sofar:",self.info.public_state.max_bet_sofar
        print "big_dealer:",self.info.public_state.dealer_id
        '''

        idx  = int(random.random() * len(self.available_actions))
        keys = self.available_actions.keys()
        '''
        print idx, len(keys)
        print keys
        print keys[idx]
        print self.available_actions[keys[idx]]
        sys.stdout.flush()
        '''

        return self.available_actions[keys[idx]]

    def reset(self):
        """

        """
        pass



#!/bin/python
import os
import roomai.common
from roomai.doudizhu.DouDiZhuPokerAction import DouDiZhuActionElement
import copy


class DouDiZhuHandCards:
    """
    """
    def __init__(self, cardstr):
        """

        Args:
            cardstr:
        """
        self.cards = [0 for i in range(DouDiZhuActionElement.total_normal_cards)]
        for c in cardstr:
            idx = DouDiZhuActionElement.str_to_rank[c]
            self.cards[idx] += 1
            if idx >= DouDiZhuActionElement.total_normal_cards:
                raise Exception("%s is invalid for a handcard" % (cardstr))

        self.num_cards = sum(self.cards)
        self.count2num = [0 for i in range(DouDiZhuActionElement.total_normal_cards)]
        for count in self.cards:
            self.count2num[count] += 1

        strs = []
        for h in range(len(self.cards)):
            for count in range(self.cards[h]):
                strs.append(DouDiZhuActionElement.rank_to_str[h])
        strs.sort()
        self.__key = "".join(strs)

    def compute_key(self):
        """

        Returns:

        """
        strs = []
        for h in range(len(self.cards)):
            for count in range(self.cards[h]):
                strs.append(DouDiZhuActionElement.rank_to_str[h])
        strs.sort()
        return "".join(strs)

    @property
    def key(self):
        """

        Returns:

        """
        return self.__key


    def add_cards(self, cards):
        """

        Args:
            cards:
        """
        if isinstance(cards, str) == True:
            cards = DouDiZhuHandCards(cards)

        for c in range(len(cards.cards)):
            count = cards.cards[c]
            self.num_cards += count
            self.count2num[self.cards[c]] -= 1
            self.cards[c] += count
            self.count2num[self.cards[c]] += 1

        self.__key = self.compute_key()


    def remove_cards(self, cards):
        """

        Args:
            cards:
        """
        if isinstance(cards, str) == True:
            cards = DouDiZhuHandCards(cards)

        for c in range(len(cards.cards)):
            count = cards.cards[c]
            self.num_cards -= count
            self.count2num[self.cards[c]] -= 1
            self.cards[c] -= count
            self.count2num[self.cards[c]] += 1

        self.__key = self.compute_key()

    def remove_action(self, action):
        """

        Args:
            action:
        """
        str = action.key
        if str == 'x' or str == 'b':
            str = ''
        self.remove_cards(DouDiZhuHandCards(str))
        self.__key = self.compute_key()

    def __deepcopy__(self, memodict={}, newinstance = None):
        """

        Args:
            memodict:
            newinstance:

        Returns:

        """
        return DouDiZhuHandCards(self.key)

class DouDiZhuPrivateState(roomai.common.AbstractPrivateState):
    """
    """
    def __init__(self):
        """

        """
        self.keep_cards = []

    def __deepcopy__(self, memodict={}, newinstance = None):
        """

        Args:
            memodict:
            newinstance:

        Returns:

        """
        if newinstance is None:
            newinstance = DouDiZhuPrivateState()
        newinstance.keep_cards = self.keep_cards.__deepcopy__()
        return newinstance


class DouDiZhuPublicState(roomai.common.AbstractPublicState):
    """
    """
    def __init__(self):
        """

        """
        self.landlord_candidate_id = -1
        self.landlord_id = -1
        self.license_playerid = -1
        self.license_action = None
        self.continuous_cheat_num = 0
        self.is_response = False

        self.keep_cards = None
        self.first_player = -1
        self.turn = -1
        self.phase = -1
        self.epoch = -1

        self.previous_id = -1
        self.previous_action = None

    def __deepcopy__(self, memodict={}, newinstance = None):
        """

        Args:
            memodict:
            newinstance:

        Returns:

        """
        if newinstance is None:
            newinstance = DouDiZhuPublicState()
        newinstance = super(DouDiZhuPublicState, self).__deepcopy__(newinstance=newinstance)

        newinstance.landlord_candidate_id = self.landlord_candidate_id
        newinstance.landlord_id = self.landlord_id
        newinstance.license_playerid = self.license_playerid
        if self.license_action is None:
            newinstance.license_action  = None
        else:
            newinstance.license_action = self.license_action.__deepcopy__()
        newinstance.continuous_cheat_num = self.continuous_cheat_num
        newinstance.is_response = self.is_response

        if self.keep_cards == None:
            newinstance.keep_cards = None
        else:
            newinstance.keep_cards = self.keep_cards.__deepcopy__()

        newinstance.first_player = self.first_player
        newinstance.turn  = self.turn
        newinstance.phase = self.phase
        newinstance.epoch = self.epoch

        newinstance.previous_id = self.previous_id
        if self.previous_action is None:
            newinstance.previous_action = None
        else:
            newinstance.previous_action = self.previous_action.__deepcopy__()

        return newinstance

class DouDiZhuPersonState(roomai.common.AbstractPersonState):
    """
    """
    def __init__(self):
        """

        """
        self.id                = None
        self.hand_cards        = None
        self.available_actions = dict()
    def __deepcopy__(self, memodict={}, newinstance = None):
        """

        Args:
            memodict:
            newinstance:

        Returns:

        """
        if newinstance is None:
            newinstance = DouDiZhuPersonState()
        newinstance.id = self.id

        if self.hand_cards is None:
            newinstance.hand_cards = None
        else:
            newinstance.hand_cards = self.hand_cards.__deepcopy__()

        if self.available_actions is None:
            newinstance.available_actions = None
        else:
            newinstance.available_actions = dict()
            for str in self.available_actions:
                newinstance.available_actions[str] = self.available_actions[str]
        return newinstance




#!/bin/python
#coding:utf-8

import roomai.common
import copy

class StageSpace:
    """
    """
    firstStage  = 1
    secondStage = 2
    thirdStage  = 3
    fourthStage = 4


class TexasHoldemAction(roomai.common.AbstractAction):
    """
    """
    # 弃牌
    Fold        = "Fold"
    # 过牌
    Check       = "Check"
    # 更注
    Call        = "Call"
    # 加注
    Raise       = "Raise"
    # all in
    AllIn       = "Allin"
    def __init__(self, key):
        """

        Args:
            key:
        """
        opt_price = key.strip().split("_")
        self.__option = opt_price[0]
        self.__price  = int(opt_price[1])
        self.__key    = "%s_%d"%(self.option, self.price)

    @property
    def key(self):
        """

        Returns:

        """
        return self.__key
    @property
    def option(self):
        """

        Returns:

        """
        return self.__option
    @property
    def price(self):
        """

        Returns:

        """
        return self.__price

    @classmethod
    def lookup(cls, key):
        """

        Args:
            key:

        Returns:

        """
        if key not in AllTexasActions:
            AllTexasActions[key] = TexasHoldemAction(key)
        return AllTexasActions[key]

    def __deepcopy__(self, memodict={}, newinstance = None):
        """

        Args:
            memodict:
            newinstance:

        Returns:

        """
        if self.key not in AllTexasActions:
            AllTexasActions[self.key] = TexasHoldemAction(self.key)
        return AllTexasActions[self.key]

AllTexasActions = dict()


class TexasHoldemPublicState(roomai.common.AbstractPublicState):
    """
    """
    def __init__(self):
        """

        """
        self.stage              = None
        self.num_players        = None
        self.dealer_id          = None
        self.public_cards       = None
        self.num_players        = None
        self.big_blind_bet      = None

        #state of players
        self.is_fold                        = None
        self.num_quit                       = None
        self.is_allin                       = None
        self.num_allin                      = None
        self.is_needed_to_action            = None
        self.num_needed_to_action           = None

        # who is expected to take a action
        self.turn               = None

        #chips is array which contains the chips of all players
        self.chips              = None

        #bets is array which contains the bets from all players
        self.bets               = None

        #max_bet = max(self.bets)
        self.max_bet_sofar      = None
        #the raise acount
        self.raise_account      = None

        self.previous_id        = None
        self.previous_action    = None

    def __deepcopy__(self, memodict={}):
            """

            Args:
                memodict:

            Returns:

            """
            copyinstance = TexasHoldemPublicState()

            copyinstance.stage         = self.stage
            copyinstance.num_players   = self.num_players
            copyinstance.dealer_id     = self.dealer_id
            copyinstance.big_blind_bet = self.big_blind_bet

            if self.public_cards is None:
                copyinstance.public_cards = None
            else:
                copyinstance.public_cards = [self.public_cards[i].__deepcopy__() for i in xrange(len(self.public_cards))]


            ######## quit, allin , needed_to_action
            copy.num_quit = self.num_quit
            if self.is_fold is None:
                copyinstance.is_fold = None
            else:
                copyinstance.is_fold = [self.is_fold[i] for i in xrange(len(self.is_fold))]

            copyinstance.num_allin = self.num_allin
            if self.is_allin is None:
                copyinstance.is_allin = None
            else:
                copyinstance.is_allin = [self.is_allin[i] for i in xrange(len(self.is_allin))]

            copyinstance.num_needed_to_action = self.num_needed_to_action
            if self.is_needed_to_action is None:
                copyinstance.is_needed_to_action = None
            else:
                copyinstance.is_needed_to_action = [self.is_needed_to_action[i] for i in
                                                    xrange(len(self.is_needed_to_action))]

            # chips is array which contains the chips of all players
            if self.chips is None:
                copyinstance.chips = None
            else:
                copyinstance.chips = [self.chips[i] for i in xrange(len(self.chips))]

            # bets is array which contains the bets from all players
            if self.bets is None:
                copyinstance.bets = None
            else:
                copyinstance.bets = [self.bets[i] for i in xrange(len(self.bets))]

            copyinstance.max_bet_sofar = self.max_bet_sofar
            copyinstance.raise_account = self.raise_account
            copyinstance.turn = self.turn

            copyinstance.previous_id = self.previous_id
            if self.previous_action is None:
                copyinstance.previous_action = None
            else:
                copyinstance.previous_action = self.previous_action.__deepcopy__()

            ### isterminal, scores
            copyinstance.is_terminal = self.is_terminal
            if self.scores is None:
                copyinstance.scores = None
            else:
                copyinstance.scores = [self.scores[i] for i in xrange(len(self.scores))]

            return copyinstance


class TexasHoldemPrivateState(roomai.common.AbstractPrivateState):
    """
    """
    keep_cards = []
    hand_cards = []

    def __deepcopy__(self, memodict={}):
        """

        Args:
            memodict:
        """
        copy = TexasHoldemPrivateState()
        if self.keep_cards is None:
            copy.keep_cards = None
        else:
            copy.keep_cards = [self.keep_cards[i].__deepcopy__() for i in xrange(len(self.keep_cards))]

        if self.hand_cards is None:
            copy.hand_cards = None
        else:
            copy.hand_cards = [[] for i in xrange(len(self.hand_cards))]
            for i in xrange(len(self.hand_cards)):
                copy.hand_cards[i] = [self.hand_cards[i][j].__deepcopy__() for j in xrange(len(self.hand_cards[i]))]


class TexasHoldemPersonState(roomai.common.AbstractPersonState):
    """
    """
    id                =    0
    hand_cards        =    []
    available_actions =    dict()

    def __deepcopy__(self, memodict={}):
        """

        Args:
            memodict:

        Returns:

        """
        copyinstance    = TexasHoldemPersonState()
        copyinstance.id = self.id
        if self.hand_cards is not None:
            copyinstance.hand_cards = [self.hand_cards[i].__deepcopy__() for i in xrange(len(self.hand_cards))]
        else:
            copyinstance.hand_cards = None

        if self.available_actions is not None:
            copyinstance.available_actions = dict()
            for key in self.available_actions:
                copyinstance.available_actions[key] = self.available_actions[key].__deepcopy__()
        else:
            copyinstance.available_actions = None
        return copyinstance


AllCardsPattern = dict()
#0     1           2       3           4                                    5     6
#name, isStraight, isPair, isSameSuit, [SizeOfPair1, SizeOfPair2,..](desc), rank, cards
AllCardsPattern["Straight_SameSuit"] = \
["Straight_SameSuit",   True,  False, True,  [],        100, []]
AllCardsPattern["4_1"] = \
["4_1",                 False, True,  False, [4,1],     98,  []]
AllCardsPattern["3_2"] = \
["3_2",                 False, True,  False, [3,2],     97,  []]
AllCardsPattern["SameSuit"] = \
["SameSuit",            False, False, True,  [],        96,  []]
AllCardsPattern["Straight_DiffSuit"] = \
["Straight_DiffSuit",   True,  False, False, [],        95,  []]
AllCardsPattern["3_1_1"] = \
["3_1_1",               False, True,  False, [3,1,1],   94,  []]
AllCardsPattern["2_2_1"] = \
["2_2_1",               False, True,  False, [2,2,1],   93,  []]
AllCardsPattern["2_1_1_1"] = \
["2_1_1_1",             False, True,  False, [2,1,1,1], 92,  []]
AllCardsPattern["1_1_1_1_1"] = \
["1_1_1_1_1",           False, True,  False, [1,1,1,1,1],91, []]







from roomai.sevenking.SevenKingUtils  import  SevenKingPokerCard
from roomai.sevenking.SevenKingUtils  import  AllSevenKingPokerCards
from roomai.sevenking.SevenKingUtils  import  AllSevenKingPatterns
from roomai.sevenking.SevenKingInfo   import  SevenKingPublicState
from roomai.sevenking.SevenKingInfo   import  SevenKingPrivateState
from roomai.sevenking.SevenKingInfo   import  SevenKingPersonState
from roomai.sevenking.SevenKingAction import  SevenKingAction
from roomai.sevenking.SevenKingEnv    import  SevenKingEnv
from roomai.sevenking.SevenKingPlayer import  SevenKingRandomPlayer


